#!/usr/bin/env python
import os, sys
import time, subprocess

# Set log path
log_path = 'logLRb/'

# Set solver paths
# Comment it if you do not run it.
solvers = {}
solvers['OPT'] = './newGLMNET/train'
solvers['CDN'] = './liblinear/train'
solvers['newGLMNET'] = './newGLMNET/train'
solvers['GLMNET'] = './runGLMNET.R'
solvers['GLMNETpath'] = './runGLMNETpath.R'
solvers['newGLMNET-ISH'] = './newGLMNET/train-innersh'
solvers['newGLMNET-OSH'] = './newGLMNET/train-outersh'
solvers['newGLMNET-NSH'] = './newGLMNET/train-nosh'

# Set data path and names
data_path = 'data/'
all_data = ['news20','rcv1','gis','yahoojp','yahookr','eps',\
			'webspam','kddb']

# The best parameter C is selected by 5-fold cross validation.
# Comment it if you do not run it.
bestC = {}
bestC[('news20','lrb')] = 64.0
bestC[('rcv1','lrb')] = 4.0
#bestC[('yahoojp','lrb')] = 4.0
#bestC[('yahookr','lrb')] = 4.0
#bestC[('gis','lrb')] = 0.25
#bestC[('eps','lrb')] = 0.5
#bestC[('webspam','lrb')] = 64.0
#bestC[('kddb','lrb')] = 0.5

# Set runtime limits (in seconds)
runtime = {}
runtime['news20'] = 5000
runtime['rcv1'] = 50000
runtime['yahoojp'] = 5000
runtime['yahookr'] = 50000
runtime['gis'] = 5000
runtime['eps'] = 50000
runtime['webspam'] = 100000
runtime['kddb'] = 100000


def run(subp, rtime):
	start = time.time()
	stop = start + rtime
	while True:
		out = subp.poll()
		if out is not None:
			return out
		if time.time() >= stop:
			print 'Already reach time limit (%s)!!' %rtime
			print 'Ready to KILL the job (%s)!!' %subp.pid
			os.kill(subp.pid, 9)
		time.sleep(3)

def compare(data):
	# Check to run or not
	if not bestC.has_key((data,'lrb')):
		print '---------------------------------------'
		print data, 'Skpped!!'
		print
		return
	# Check if data exist
	train_data = os.path.join(data_path, '%s.train'%data)
	test_data = os.path.join(data_path, '%s.test'%data)
	if not os.path.exists(train_data) or\
	   not os.path.exists(test_data):
		print '---------------------------------------'
		print train_data, 'or', test_data, 'do not exist!!'
		print
		return
	# Set default runtime 1000
	if not runtime.has_key(data):
		runtime[data] = 1000

	cmd = 'wc -l %s' %train_data
	fp = os.popen(cmd)
	no_inst = int(fp.readline().strip().split()[0])
	fp.close()

	C = bestC[(data,'lrb')]
	lambda1 = 1.0/C
	lambda2 = 1.0/C/no_inst
	print '---------------------------------------'
	print train_data, no_inst
	print C, lambda1, lambda2
	print '---------------------------------------'

	#OPT
	if solvers.has_key('OPT'):
		solver = solvers['OPT']
		eps = 1e-8
		log = os.path.join(log_path,'OPT_%s_%s.log'%(data,C))
		tmplog = '%s.tmp' %log
		cmd = '%s -s 6 -c %s -e %s -n 1 -g 1 %s %s'\
			%(solver,C,eps,train_data,test_data)
		if os.path.exists(solver) and not os.path.exists(log):
			print 'OPT Running...'
			print cmd
			fobj = open(tmplog, 'w')
			subp = subprocess.Popen(cmd.split(), shell=False, stdout=fobj)
			run(subp, runtime[data]*5)
			fobj.close()
			os.system('mv %s %s' %(tmplog,log))
		else:
			print 'OPT Skipped!!'
	else:
		print 'OPT Skipped!!'

	#CDN
	if solvers.has_key('CDN'):
		solver = solvers['CDN']
		eps = 16.0
		log = os.path.join(log_path,'CDN_%s_%s.log'%(data,C))
		tmplog = '%s.tmp' %log
		cmd = '%s -s 7 -c %s -e %s -n 80 -g 1.2 %s %s'\
			%(solver,C,eps,train_data,test_data)
		if os.path.exists(solver) and not os.path.exists(log):
			print 'CDN Running...'
			print cmd
			fobj = open(tmplog, 'w')
			subp = subprocess.Popen(cmd.split(), shell=False, stdout=fobj)
			run(subp, runtime[data]*5)
			fobj.close()
			os.system('mv %s %s' %(tmplog,log))
		else:
			print 'CDN Skipped!!'
	else:
		print 'CDN Skipped!!'

	#newGLMNET
	if solvers.has_key('newGLMNET'):
		solver = solvers['newGLMNET']
		eps = 16.0
		log = os.path.join(log_path,'newGLMNET_%s_%s.log'%(data,C))
		tmplog = '%s.tmp' %log
		cmd = '%s -s 6 -c %s -e %s -n 80 -g 1.2 %s %s'\
			%(solver,C,eps,train_data,test_data)
		if os.path.exists(solver) and not os.path.exists(log):
			print 'newGLMNET Running...'
			print cmd
			fobj = open(tmplog, 'w')
			subp = subprocess.Popen(cmd.split(), shell=False, stdout=fobj)
			run(subp, runtime[data]*5)
			fobj.close()
			os.system('mv %s %s' %(tmplog,log))
		else:
			print 'newGLMNET Skipped!!'
	else:
		print 'newGLMNET Skipped!!'

	#GLMNET
	if solvers.has_key('GLMNET'):
		solver = solvers['GLMNET']
		eps = 1e-4
		max_iter = 100
		log = os.path.join(log_path,'GLMNET_%s_%s.log'%(data,C))
		tmplog = '%s.tmp' %log
		Rmsg = './tmp/GLMNET_%s_%s.Rmsg' %(data,C)
		arg = "--vanilla --slave --args C=%s eps=%s iter=%s data=\"%s\""\
			%(C,eps,max_iter,os.path.join(data_path,data))
		if os.path.exists(solver) and not os.path.exists(log):
			print 'GLMNET Running...'
			print 'R', arg, '<', solver
			fin = open(solver, 'r')
			fout = open(tmplog, 'w')
			subp = subprocess.Popen(['R',arg],shell=False, stdin=fin,stdout=fout)
			run(subp, runtime[data])
			fin.close()
			fout.close()
			os.system('mv %s %s' %(tmplog,log))
		else:
			print 'GLMNET Skipped!!'
	else:
		print 'GLMNET Skipped!!'

	#GLMNETpath
	if solvers.has_key('GLMNETpath'):
		solver = solvers['GLMNETpath']
		eps = 1e-4
		max_iter = 100
		log = os.path.join(log_path,'GLMNETpath_%s_%s.log'%(data,C))
		tmplog = '%s.tmp' %log
		Rmsg = './tmp/GLMNETpath_%s_%s.Rmsg' %(data,C)
		arg = "--vanilla --slave --args C=%s eps=%s iter=%s data=\"%s\""\
			%(C,eps,max_iter,os.path.join(data_path,data))
		if os.path.exists(solver) and not os.path.exists(log):
			print 'GLMNETpath Running...'
			print 'R', arg, '<', solver
			fin = open(solver, 'r')
			fout = open(tmplog, 'w')
			subp = subprocess.Popen(['R',arg],shell=False, stdin=fin,stdout=fout)
			run(subp, runtime[data])
			fin.close()
			fout.close()
			os.system('mv %s %s' %(tmplog,log))
		else:
			print 'GLMNETpath Skipped!!'
	else:
		print 'GLMNETpath Skipped!!'

	#newGLMNET-ISH
	if solvers.has_key('newGLMNET-ISH'):
		solver = solvers['newGLMNET-ISH']
		eps = 16.0
		log = os.path.join(log_path,'newGLMNET-ISH_%s_%s.log'%(data,C))
		tmplog = '%s.tmp' %log
		cmd = '%s -s 6 -c %s -e %s -n 80 -g 1.2 %s %s'\
			%(solver,C,eps,train_data,test_data)
		if os.path.exists(solver) and not os.path.exists(log):
			print 'newGLMNET-ISH Running...'
			print cmd
			fobj = open(tmplog, 'w')
			subp = subprocess.Popen(cmd.split(), shell=False, stdout=fobj)
			run(subp, runtime[data]*5)
			fobj.close()
			os.system('mv %s %s' %(tmplog,log))
		else:
			print 'newGLMNET-ISH Skipped!!'
	else:
		print 'newGLMNET-ISH Skipped!!'

	#newGLMNET-OSH
	if solvers.has_key('newGLMNET-OSH'):
		solver = solvers['newGLMNET-OSH']
		eps = 16.0
		log = os.path.join(log_path,'newGLMNET-OSH_%s_%s.log'%(data,C))
		tmplog = '%s.tmp' %log
		cmd = '%s -s 6 -c %s -e %s -n 80 -g 1.2 %s %s'\
			%(solver,C,eps,train_data,test_data)
		if os.path.exists(solver) and not os.path.exists(log):
			print 'newGLMNET-OSH Running...'
			print cmd
			fobj = open(tmplog, 'w')
			subp = subprocess.Popen(cmd.split(), shell=False, stdout=fobj)
			run(subp, runtime[data]*5)
			fobj.close()
			os.system('mv %s %s' %(tmplog,log))
		else:
			print 'newGLMNET-OSH Skipped!!'
	else:
		print 'newGLMNET-OSH Skipped!!'

	#newGLMNET-NSH
	if solvers.has_key('newGLMNET-NSH'):
		solver = solvers['newGLMNET-NSH']
		eps = 16.0
		log = os.path.join(log_path,'newGLMNET-NSH_%s_%s.log'%(data,C))
		tmplog = '%s.tmp' %log
		cmd = '%s -s 6 -c %s -e %s -n 80 -g 1.2 %s %s'\
			%(solver,C,eps,train_data,test_data)
		if os.path.exists(solver) and not os.path.exists(log):
			print 'newGLMNET-NSH Running...'
			print cmd
			fobj = open(tmplog, 'w')
			subp = subprocess.Popen(cmd.split(), shell=False, stdout=fobj)
			run(subp, runtime[data]*5)
			fobj.close()
			os.system('mv %s %s' %(tmplog,log))
		else:
			print 'newGLMNET-NSH Skipped!!'
	else:
		print 'newGLMNET-NSH Skipped!!'


if __name__ == '__main__':
	data_list = []
	for data in all_data:
		if (data,'lrb') in bestC.keys():
			data_list.append(data)
	for data in data_list:
		compare(data)

