#!/usr/bin/env Rscript
.libPaths(c("./lib",.libPaths()))
library("SparseM")
library("Matrix")
library("e1071")
library("glmnet")

C = 2.0
eps = 1.0e-6
iter = 100
data = "./data/a9a"
################
eps_g = 2.0
eps_n = 1

args = commandArgs(TRUE)
if(length(args) != 0) {
	for(i in 1:length(args)) {
		eval(parse(text=args[[i]]))
	}
} else {
	print("R --vanilla --slave --args C=2.0 eps=1.0e-6 iter=100 data='./data/a9a' eps_g=2.0 eps_n=1 < runGLMNETpath.R > runGLMNETpath.Rout")
}

print("Input arguments:")
cat("C",C,"\n")
cat("eps",eps,"\n")
cat("iter",iter,"\n")
cat("data",data,"\n")
cat("eps_g",eps_g,"\n")
cat("eps_n",eps_n,"\n")

traindata = paste(data,"train",sep=".")
testdata = paste(data,"test",sep=".")

#load training data
cat("Training Data:",traindata,"\n")
print("loading...")
svm = read.matrix.csr(traindata)
X = svm[["x"]]
X = as(X,"Matrix")
dimX = dim(X)
Y = svm[["y"]]
Y = c(Y)
lenY = length(Y)
cat("dimX",dimX,"\n")
cat("lenY",lenY,"\n")
rm(svm)

#load testing data
cat("Testing Data:",testdata,"\n")
print("loading...")
svm = read.matrix.csr(testdata,fac=TRUE,ncol=dimX[2])
XX = svm[["x"]]
XX = as(XX,"Matrix")
dimXX = dim(XX)
YY = svm[["y"]]
YY = c(YY)
lenYY = length(YY)
cat("dimXX",dimXX,"\n")
cat("lenYY",lenYY,"\n")
rm(svm)

# lambda = 1/C/l
lam = 1.0/C/lenY
cat("C",C, "l",lenY,"\n")
cat("lambda",lam,"\n")

print("Training ...")
for(i in 1:eps_n) {
	cat("i",i, "eps",eps,"\n")
	model = glmnet(X,Y,XX,YY,family="binomial",lambda.min.ratio=lam,standardize=FALSE,thresh=eps,maxit=iter)
	eps = eps / eps_g
	print(model)
	rm(model)
}

